@@ -36,8 +36,7 @@ bool AutoScriptEvaluate::StartEvaluating(JSObject *scope, JSErrorReporter errorR
     }
 
     JS_BeginRequest(mJSContext);
-    if (!mEnterCompartment.enter(mJSContext, scope))
-        return false;
+    mAutoCompartment.construct(mJSContext, scope);
 
     // Saving the exception state keeps us from interfering with another script
     // that may also be running on this context.  This occurred first with the
@@ -101,9 +100,9 @@ JSBool xpc_IsReportableErrorCode(nsresult code)
         case NS_ERROR_FACTORY_REGISTER_AGAIN:
         case NS_BASE_STREAM_WOULD_BLOCK:
             return false;
+        default:
+            return true;
     }
-
-    return true;
 }
 
 // static
@@ -111,7 +110,7 @@ nsresult
 nsXPCWrappedJSClass::GetNewOrUsed(XPCCallContext& ccx, REFNSIID aIID,
                                   nsXPCWrappedJSClass** resultClazz)
 {
-    nsXPCWrappedJSClass* clazz = nsnull;
+    nsXPCWrappedJSClass* clazz = nullptr;
     XPCJSRuntime* rt = ccx.GetRuntime();
 
     {   // scoped lock
@@ -131,7 +130,7 @@ nsXPCWrappedJSClass::GetNewOrUsed(XPCCallContext& ccx, REFNSIID aIID,
                 nsXPConnect::IsISupportsDescendant(info)) {
                 clazz = new nsXPCWrappedJSClass(ccx, aIID, info);
                 if (clazz && !clazz->mDescriptors)
-                    NS_RELEASE(clazz);  // sets clazz to nsnull
+                    NS_RELEASE(clazz);  // sets clazz to nullptr
             }
         }
     }
@@ -143,9 +142,9 @@ nsXPCWrappedJSClass::nsXPCWrappedJSClass(XPCCallContext& ccx, REFNSIID aIID,
                                          nsIInterfaceInfo* aInfo)
     : mRuntime(ccx.GetRuntime()),
       mInfo(aInfo),
-      mName(nsnull),
+      mName(nullptr),
       mIID(aIID),
-      mDescriptors(nsnull)
+      mDescriptors(nullptr)
 {
     NS_ADDREF(mInfo);
     NS_ADDREF_THIS();
@@ -159,7 +158,7 @@ nsXPCWrappedJSClass::nsXPCWrappedJSClass(XPCCallContext& ccx, REFNSIID aIID,
     if (NS_SUCCEEDED(mInfo->GetMethodCount(&methodCount))) {
         if (methodCount) {
             int wordCount = (methodCount/32)+1;
-            if (nsnull != (mDescriptors = new uint32_t[wordCount])) {
+            if (nullptr != (mDescriptors = new uint32_t[wordCount])) {
                 int i;
                 // init flags to 0;
                 for (i = wordCount-1; i >= 0; i--)
@@ -171,7 +170,7 @@ nsXPCWrappedJSClass::nsXPCWrappedJSClass(XPCCallContext& ccx, REFNSIID aIID,
                         SetReflectable(i, XPCConvert::IsMethodReflectable(*info));
                     else {
                         delete [] mDescriptors;
-                        mDescriptors = nsnull;
+                        mDescriptors = nullptr;
                         break;
                     }
                 }
@@ -214,7 +213,7 @@ nsXPCWrappedJSClass::CallQueryInterfaceOnJSObject(XPCCallContext& ccx,
     // interface (i.e. whether the interface is scriptable) and most content
     // objects don't have QI implementations anyway. Also see bug 503926.
     if (!xpc::AccessCheck::isChrome(js::GetObjectCompartment(jsobj))) {
-        return nsnull;
+        return nullptr;
     }
 
     // OK, it looks like we'll be calling into JS code.
@@ -223,12 +222,12 @@ nsXPCWrappedJSClass::CallQueryInterfaceOnJSObject(XPCCallContext& ccx,
     // XXX we should install an error reporter that will send reports to
     // the JS error console service.
     if (!scriptEval.StartEvaluating(jsobj))
-        return nsnull;
+        return nullptr;
 
     // check upfront for the existence of the function property
     funid = mRuntime->GetStringID(XPCJSRuntime::IDX_QUERY_INTERFACE);
     if (!JS_GetPropertyById(cx, jsobj, funid, &fun) || JSVAL_IS_PRIMITIVE(fun))
-        return nsnull;
+        return nullptr;
 
     // protect fun so that we're sure it's alive when we call it
     AUTO_MARK_JSVAL(ccx, fun);
@@ -243,11 +242,11 @@ nsXPCWrappedJSClass::CallQueryInterfaceOnJSObject(XPCCallContext& ccx,
         nsCOMPtr<nsIInterfaceInfo> info;
         ccx.GetXPConnect()->GetInfoForIID(&aIID, getter_AddRefs(info));
         if (!info)
-            return nsnull;
+            return nullptr;
         bool canScript, isBuiltin;
         if (NS_FAILED(info->IsScriptable(&canScript)) || !canScript ||
             NS_FAILED(info->IsBuiltinClass(&isBuiltin)) || isBuiltin)
-            return nsnull;
+            return nullptr;
     }
 
     id = xpc_NewIDObject(cx, jsobj, aIID);
@@ -313,7 +312,7 @@ nsXPCWrappedJSClass::CallQueryInterfaceOnJSObject(XPCCallContext& ccx,
     if (success)
         success = JS_ValueToObject(cx, retval, &retObj);
 
-    return success ? retObj : nsnull;
+    return success ? retObj : nullptr;
 }
 
 /***************************************************************************/
@@ -412,7 +411,7 @@ nsXPCWrappedJSClass::BuildPropertyEnumerator(XPCCallContext& ccx,
             return NS_ERROR_FAILURE;
 
         nsCOMPtr<nsIProperty> property =
-            new xpcProperty(chars, (PRUint32) length, value);
+            new xpcProperty(chars, (uint32_t) length, value);
 
         if (!propertyArray.AppendObject(property))
             return NS_ERROR_FAILURE;
@@ -425,7 +424,7 @@ nsXPCWrappedJSClass::BuildPropertyEnumerator(XPCCallContext& ccx,
 
 NS_IMPL_ISUPPORTS1(xpcProperty, nsIProperty)
 
-xpcProperty::xpcProperty(const PRUnichar* aName, PRUint32 aNameLen,
+xpcProperty::xpcProperty(const PRUnichar* aName, uint32_t aNameLen,
                          nsIVariant* aValue)
     : mName(aName, aNameLen), mValue(aValue)
 {
@@ -467,7 +466,7 @@ public:
 
     static void* GetSingleton()
     {
-        static WrappedJSIdentity* singleton = nsnull;
+        static WrappedJSIdentity* singleton = nullptr;
         if (!singleton)
             singleton = new WrappedJSIdentity();
         return (void*) singleton;
@@ -497,16 +496,14 @@ GetContextFromObject(JSObject *obj)
     XPCJSContextStack* stack = XPCJSRuntime::Get()->GetJSContextStack();
 
     if (stack && stack->Peek())
-        return nsnull;
+        return nullptr;
 
     // In order to get a context, we need a context.
     XPCCallContext ccx(NATIVE_CALLER);
     if (!ccx.IsValid())
-        return nsnull;
+        return nullptr;
 
-    JSAutoEnterCompartment ac;
-    if (!ac.enter(ccx, obj))
-        return nsnull;
+    JSAutoCompartment ac(ccx, obj);
     XPCWrappedNativeScope* scope =
         XPCWrappedNativeScope::FindInJSObjectScope(ccx, obj);
     XPCContext *xpcc = scope->GetContext();
@@ -517,7 +514,7 @@ GetContextFromObject(JSObject *obj)
         return cx;
     }
 
-    return nsnull;
+    return nullptr;
 }
 
 class SameOriginCheckedComponent MOZ_FINAL : public nsISecurityCheckedComponent
@@ -543,7 +540,7 @@ NS_INTERFACE_MAP_END_AGGREGATED(mDelegate)
 
 NS_IMETHODIMP
 SameOriginCheckedComponent::CanCreateWrapper(const nsIID * iid,
-                                             char **_retval NS_OUTPARAM)
+                                             char **_retval)
 {
     // XXX This doesn't actually work because nsScriptSecurityManager doesn't
     // know what to do with "sameOrigin" for canCreateWrapper.
@@ -554,7 +551,7 @@ SameOriginCheckedComponent::CanCreateWrapper(const nsIID * iid,
 NS_IMETHODIMP
 SameOriginCheckedComponent::CanCallMethod(const nsIID * iid,
                                           const PRUnichar *methodName,
-                                          char **_retval NS_OUTPARAM)
+                                          char **_retval)
 {
     *_retval = NS_strdup("sameOrigin");
     return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
@@ -563,7 +560,7 @@ SameOriginCheckedComponent::CanCallMethod(const nsIID * iid,
 NS_IMETHODIMP
 SameOriginCheckedComponent::CanGetProperty(const nsIID * iid,
                                            const PRUnichar *propertyName,
-                                           char **_retval NS_OUTPARAM)
+                                           char **_retval)
 {
     *_retval = NS_strdup("sameOrigin");
     return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
@@ -572,7 +569,7 @@ SameOriginCheckedComponent::CanGetProperty(const nsIID * iid,
 NS_IMETHODIMP
 SameOriginCheckedComponent::CanSetProperty(const nsIID * iid,
                                            const PRUnichar *propertyName,
-                                           char **_retval NS_OUTPARAM)
+                                           char **_retval)
 {
     *_retval = NS_strdup("sameOrigin");
     return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
@@ -602,7 +599,7 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
         nsXPCWrappedJS* root = self->GetRootWrapper();
 
         if (!root->IsValid()) {
-            *aInstancePtr = nsnull;
+            *aInstancePtr = nullptr;
             return NS_NOINTERFACE;
         }
 
@@ -613,14 +610,14 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
 
     // We can't have a cached wrapper.
     if (aIID.Equals(NS_GET_IID(nsWrapperCache))) {
-        *aInstancePtr = nsnull;
+        *aInstancePtr = nullptr;
         return NS_NOINTERFACE;
     }
 
     JSContext *context = GetContextFromObject(self->GetJSObject());
     XPCCallContext ccx(NATIVE_CALLER, context);
     if (!ccx.IsValid()) {
-        *aInstancePtr = nsnull;
+        *aInstancePtr = nullptr;
         return NS_NOINTERFACE;
     }
 
@@ -633,7 +630,7 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
         // Fail if JSObject doesn't claim support for nsISupportsWeakReference
         if (!root->IsValid() ||
             !CallQueryInterfaceOnJSObject(ccx, root->GetJSObject(), aIID)) {
-            *aInstancePtr = nsnull;
+            *aInstancePtr = nullptr;
             return NS_NOINTERFACE;
         }
 
@@ -647,14 +644,14 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
     // Checks for any existing wrapper explicitly constructed for this iid.
     // This includes the current 'self' wrapper. This also deals with the
     // nsISupports case (for which it returns mRoot).
-    if (nsnull != (sibling = self->Find(aIID))) {
+    if (nullptr != (sibling = self->Find(aIID))) {
         NS_ADDREF(sibling);
         *aInstancePtr = sibling->GetXPTCStub();
         return NS_OK;
     }
 
     // Check if asking for an interface from which one of our wrappers inherits.
-    if (nsnull != (sibling = self->FindInherited(aIID))) {
+    if (nullptr != (sibling = self->FindInherited(aIID))) {
         NS_ADDREF(sibling);
         *aInstancePtr = sibling->GetXPTCStub();
         return NS_OK;
@@ -679,7 +676,7 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
         // known as system) principals. It really wants to do a
         // UniversalXPConnect type check.
 
-        *aInstancePtr = nsnull;
+        *aInstancePtr = nullptr;
 
         nsXPConnect *xpc = nsXPConnect::GetXPConnect();
         nsCOMPtr<nsIScriptSecurityManager> secMan =
@@ -727,7 +724,7 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
         // XPConvert::JSObject2NativeInterface() here to make sure we
         // get a new (or used) nsXPCWrappedJS.
         nsXPCWrappedJS* wrapper;
-        nsresult rv = nsXPCWrappedJS::GetNewOrUsed(ccx, jsobj, aIID, nsnull,
+        nsresult rv = nsXPCWrappedJS::GetNewOrUsed(ccx, jsobj, aIID, nullptr,
                                                    &wrapper);
         if (NS_SUCCEEDED(rv) && wrapper) {
             // We need to go through the QueryInterface logic to make
@@ -741,7 +738,7 @@ nsXPCWrappedJSClass::DelegatedQueryInterface(nsXPCWrappedJS* self,
 
     // else...
     // no can do
-    *aInstancePtr = nsnull;
+    *aInstancePtr = nullptr;
     return NS_NOINTERFACE;
 }
 
@@ -815,7 +812,7 @@ xpcWrappedJSErrorReporter(JSContext *cx, const char *message,
         return;
 
     nsCOMPtr<nsIException> e;
-    XPCConvert::JSErrorToXPCException(ccx, message, nsnull, nsnull, report,
+    XPCConvert::JSErrorToXPCException(ccx, message, nullptr, nullptr, report,
                                       getter_AddRefs(e));
     if (e)
         ccx.GetXPCContext()->SetException(e);
@@ -954,7 +951,7 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
 
     xpcc->GetException(getter_AddRefs(xpc_exception));
     if (xpc_exception)
-        xpcc->SetException(nsnull);
+        xpcc->SetException(nullptr);
 
     // get this right away in case we do something below to cause JS code
     // to run on this JSContext
@@ -972,7 +969,7 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
 
         /* cleanup and set failed even if we can't build an exception */
         if (!xpc_exception) {
-            XPCJSRuntime::Get()->SetPendingException(nsnull); // XXX necessary?
+            XPCJSRuntime::Get()->SetPendingException(nullptr); // XXX necessary?
         }
     }
 
@@ -998,7 +995,7 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
                 // Finally, check to see if this is the last JS frame on the
                 // stack. If so then we always want to report it.
                 if (!reportable) {
-                    reportable = !JS_DescribeScriptedCaller(cx, nsnull, nsnull);
+                    reportable = !JS_DescribeScriptedCaller(cx, nullptr, nullptr);
                 }
 
                 // Ugly special case for GetInterface. It's "special" in the
@@ -1045,7 +1042,7 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
                 // something with it.
                 nsCOMPtr<nsIConsoleService> consoleService
                     (do_GetService(XPC_CONSOLE_CONTRACTID));
-                if (nsnull != consoleService) {
+                if (nullptr != consoleService) {
                     nsresult rv;
                     nsCOMPtr<nsIScriptError> scriptError;
                     nsCOMPtr<nsISupports> errorData;
@@ -1053,11 +1050,11 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
                     if (NS_SUCCEEDED(rv))
                         scriptError = do_QueryInterface(errorData);
 
-                    if (nsnull == scriptError) {
+                    if (nullptr == scriptError) {
                         // No luck getting one from the exception, so
                         // try to cook one up.
                         scriptError = do_CreateInstance(XPC_SCRIPT_ERROR_CONTRACTID);
-                        if (nsnull != scriptError) {
+                        if (nullptr != scriptError) {
                             char* exn_string;
                             rv = xpc_exception->ToString(&exn_string);
                             if (NS_SUCCEEDED(rv)) {
@@ -1067,7 +1064,7 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
 
                                 // try to get filename, lineno from the first
                                 // stack frame location.
-                                PRInt32 lineNumber = 0;
+                                int32_t lineNumber = 0;
                                 nsXPIDLCString sourceName;
 
                                 nsCOMPtr<nsIStackFrame> location;
@@ -1083,16 +1080,16 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
 
                                 rv = scriptError->InitWithWindowID(newMessage.get(),
                                                                    NS_ConvertASCIItoUTF16(sourceName).get(),
-                                                                   nsnull,
+                                                                   nullptr,
                                                                    lineNumber, 0, 0,
                                                                    "XPConnect JavaScript",
                                                                    nsJSUtils::GetCurrentlyRunningCodeInnerWindowID(cx));
                                 if (NS_FAILED(rv))
-                                    scriptError = nsnull;
+                                    scriptError = nullptr;
                             }
                         }
                     }
-                    if (nsnull != scriptError)
+                    if (nullptr != scriptError)
                         consoleService->LogMessage(scriptError);
                 }
             }
@@ -1114,17 +1111,18 @@ nsXPCWrappedJSClass::CheckForException(XPCCallContext & ccx,
 
 NS_IMETHODIMP
 nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16_t methodIndex,
-                                const XPTMethodDescriptor* info,
+                                const XPTMethodDescriptor* info_,
                                 nsXPTCMiniVariant* nativeParams)
 {
-    jsval* sp = nsnull;
-    jsval* argv = nsnull;
+    jsval* sp = nullptr;
+    jsval* argv = nullptr;
     uint8_t i;
     nsresult retval = NS_ERROR_FAILURE;
     nsresult pending_result = NS_OK;
     JSBool success;
     JSBool readyToDoTheCall = false;
     nsID  param_iid;
+    const nsXPTMethodInfo* info = static_cast<const nsXPTMethodInfo*>(info_);
     const char* name = info->name;
     jsval fval;
     JSBool foundDependentParam;
@@ -1144,13 +1142,21 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16_t methodIndex,
     if (!cx || !xpcc || !IsReflectable(methodIndex))
         return NS_ERROR_FAILURE;
 
+    // [implicit_jscontext] and [optional_argc] have a different calling
+    // convention, which we don't support for JS-implemented components.
+    if (info->WantsOptArgc() || info->WantsContext()) {
+        const char *str = "IDL methods marked with [implicit_jscontext] "
+                          "or [optional_argc] may not be implemented in JS";
+        // Throw and warn for good measure.
+        JS_ReportError(cx, str);
+        NS_WARNING(str);
+        return NS_ERROR_FAILURE;
+    }
+
     JSObject *obj = wrapper->GetJSObject();
     JSObject *thisObj = obj;
 
-    JSAutoEnterCompartment ac;
-    if (!ac.enter(cx, obj))
-        return NS_ERROR_FAILURE;
-
+    JSAutoCompartment ac(cx, obj);
     ccx.SetScopeForNewJSObjects(obj);
 
     JS::AutoValueVector args(cx);
@@ -1165,8 +1171,8 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16_t methodIndex,
         goto pre_call_clean_up;
 
     xpcc->SetPendingResult(pending_result);
-    xpcc->SetException(nsnull);
-    XPCJSRuntime::Get()->SetPendingException(nsnull);
+    xpcc->SetException(nullptr);
+    XPCJSRuntime::Get()->SetPendingException(nullptr);
 
     // We use js_Invoke so that the gcthings we use as args will be rooted by
     // the engine as we do conversions and prepare to do the function call.
@@ -1221,7 +1227,7 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16_t methodIndex,
 
                         if (translator) {
                             bool hideFirstParamFromJS = false;
-                            nsIID* newWrapperIID = nsnull;
+                            nsIID* newWrapperIID = nullptr;
                             nsCOMPtr<nsISupports> newThis;
 
                             if (NS_FAILED(translator->
@@ -1241,8 +1247,8 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16_t methodIndex,
                                 xpcObjectHelper helper(newThis);
                                 JSBool ok =
                                   XPCConvert::NativeInterface2JSObject(ccx,
-                                                                       &v, nsnull, helper, newWrapperIID,
-                                                                       nsnull, false, nsnull);
+                                                                       &v, nullptr, helper, newWrapperIID,
+                                                                       nullptr, false, nullptr);
                                 if (newWrapperIID)
                                     nsMemory::Free(newWrapperIID);
                                 if (!ok) {
@@ -1332,17 +1338,17 @@ nsXPCWrappedJSClass::CallMethod(nsXPCWrappedJS* wrapper, uint16_t methodIndex,
                 if (!XPCConvert::NativeArray2JS(lccx, &val,
                                                 (const void**)&pv->val,
                                                 datum_type, &param_iid,
-                                                array_count, nsnull))
+                                                array_count, nullptr))
                     goto pre_call_clean_up;
             } else if (isSizedString) {
                 if (!XPCConvert::NativeStringWithSize2JS(ccx, &val,
                                                          (const void*)&pv->val,
                                                          datum_type,
-                                                         array_count, nsnull))
+                                                         array_count, nullptr))
                     goto pre_call_clean_up;
             } else {
                 if (!XPCConvert::NativeData2JS(ccx, &val, &pv->val, type,
-                                               &param_iid, nsnull))
+                                               &param_iid, nullptr))
                     goto pre_call_clean_up;
             }
         }
@@ -1386,7 +1392,7 @@ pre_call_clean_up:
         if (param.IsIn()) {
             if (type.IsArray()) {
                 void** pp;
-                if (nsnull != (pp = *((void***)p))) {
+                if (nullptr != (pp = *((void***)p))) {
 
                     // we need to get the array length and iterate the items
                     uint32_t array_count;
@@ -1408,7 +1414,7 @@ pre_call_clean_up:
             } else
                 CleanupPointerTypeObject(type, (void**)p);
         }
-        *((void**)p) = nsnull;
+        *((void**)p) = nullptr;
     }
 
     // Make sure "this" doesn't get deleted during this call.
@@ -1444,15 +1450,15 @@ pre_call_clean_up:
                     NS_ERROR_XPC_JSOBJECT_HAS_NO_FUNCTION_NAMED;
             static const char format[] = "%s \"%s\"";
             const char * msg;
-            char* sz = nsnull;
+            char* sz = nullptr;
 
-            if (nsXPCException::NameAndFormatForNSResult(code, nsnull, &msg) && msg)
+            if (nsXPCException::NameAndFormatForNSResult(code, nullptr, &msg) && msg)
                 sz = JS_smprintf(format, msg, name);
 
             nsCOMPtr<nsIException> e;
 
             XPCConvert::ConstructException(code, sz, GetInterfaceName(), name,
-                                           nsnull, getter_AddRefs(e), nsnull, nsnull);
+                                           nullptr, getter_AddRefs(e), nullptr, nullptr);
             xpcc->SetException(e);
             if (sz)
                 JS_smprintf_free(sz);
@@ -1471,7 +1477,7 @@ pre_call_clean_up:
         return CheckForException(ccx, name, GetInterfaceName(), forceReport);
     }
 
-    XPCJSRuntime::Get()->SetPendingException(nsnull); // XXX necessary?
+    XPCJSRuntime::Get()->SetPendingException(nullptr); // XXX necessary?
 
     // convert out args and result
     // NOTE: this is the total number of native params, not just the args
@@ -1520,7 +1526,7 @@ pre_call_clean_up:
         }
 
         if (!XPCConvert::JSData2Native(ccx, &pv->val, val, type,
-                                       !param.IsDipper(), &param_iid, nsnull))
+                                       !param.IsDipper(), &param_iid, nullptr))
             break;
     }
 
@@ -1580,18 +1586,18 @@ pre_call_clean_up:
                 if (array_count &&
                     !XPCConvert::JSArray2Native(ccx, (void**)&pv->val, val,
                                                 array_count, datum_type,
-                                                &param_iid, nsnull))
+                                                &param_iid, nullptr))
                     break;
             } else if (isSizedString) {
                 if (!XPCConvert::JSStringWithSize2Native(ccx,
                                                          (void*)&pv->val, val,
                                                          array_count, datum_type,
-                                                         nsnull))
+                                                         nullptr))
                     break;
             } else {
                 if (!XPCConvert::JSData2Native(ccx, &pv->val, val, type,
                                                true, &param_iid,
-                                               nsnull))
+                                               nullptr))
                     break;
             }
         }
@@ -1614,7 +1620,7 @@ pre_call_clean_up:
 
             if (type.IsArray()) {
                 void** pp;
-                if (nsnull != (pp = *((void***)p))) {
+                if (nullptr != (pp = *((void***)p))) {
                     // we need to get the array length and iterate the items
                     uint32_t array_count;
                     nsXPTType datum_type;
@@ -1632,7 +1638,7 @@ pre_call_clean_up:
                 }
             } else
                 CleanupPointerTypeObject(type, (void**)p);
-            *((void**)p) = nsnull;
+            *((void**)p) = nullptr;
         }
     } else {
         // set to whatever the JS code might have set as the result
@@ -1653,12 +1659,12 @@ nsXPCWrappedJSClass::GetInterfaceName()
 JSObject*
 nsXPCWrappedJSClass::NewOutObject(JSContext* cx, JSObject* scope)
 {
-    return JS_NewObject(cx, nsnull, nsnull, JS_GetGlobalForObject(cx, scope));
+    return JS_NewObject(cx, nullptr, nullptr, JS_GetGlobalForObject(cx, scope));
 }
 
 
 NS_IMETHODIMP
-nsXPCWrappedJSClass::DebugDump(PRInt16 depth)
+nsXPCWrappedJSClass::DebugDump(int16_t depth)
 {
 #ifdef DEBUG
     depth-- ;
