@@ -28,14 +28,19 @@
 #include "nsReadLine.h"
 #include "nsNetUtil.h"
 #include "nsIClassInfoImpl.h"
+#include "mozilla/ipc/InputStreamUtils.h"
 
 #define NS_NO_INPUT_BUFFERING 1 // see http://bugzilla.mozilla.org/show_bug.cgi?id=41067
 
+typedef mozilla::ipc::FileDescriptor::PlatformHandleType FileHandleType;
+
+using namespace mozilla::ipc;
+
 ////////////////////////////////////////////////////////////////////////////////
 // nsFileStreamBase
 
 nsFileStreamBase::nsFileStreamBase()
-    : mFD(nsnull)
+    : mFD(nullptr)
     , mBehaviorFlags(0)
     , mDeferredOpen(false)
 {
@@ -49,32 +54,32 @@ nsFileStreamBase::~nsFileStreamBase()
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsFileStreamBase, nsISeekableStream)
 
 NS_IMETHODIMP
-nsFileStreamBase::Seek(PRInt32 whence, PRInt64 offset)
+nsFileStreamBase::Seek(int32_t whence, int64_t offset)
 {
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (mFD == nsnull)
+    if (mFD == nullptr)
         return NS_BASE_STREAM_CLOSED;
 
-    PRInt64 cnt = PR_Seek64(mFD, offset, (PRSeekWhence)whence);
-    if (cnt == PRInt64(-1)) {
+    int64_t cnt = PR_Seek64(mFD, offset, (PRSeekWhence)whence);
+    if (cnt == int64_t(-1)) {
         return NS_ErrorAccordingToNSPR();
     }
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileStreamBase::Tell(PRInt64 *result)
+nsFileStreamBase::Tell(int64_t *result)
 {
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (mFD == nsnull)
+    if (mFD == nullptr)
         return NS_BASE_STREAM_CLOSED;
 
-    PRInt64 cnt = PR_Seek64(mFD, 0, PR_SEEK_CUR);
-    if (cnt == PRInt64(-1)) {
+    int64_t cnt = PR_Seek64(mFD, 0, PR_SEEK_CUR);
+    if (cnt == int64_t(-1)) {
         return NS_ErrorAccordingToNSPR();
     }
     *result = cnt;
@@ -87,12 +92,12 @@ nsFileStreamBase::SetEOF()
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (mFD == nsnull)
+    if (mFD == nullptr)
         return NS_BASE_STREAM_CLOSED;
 
 #if defined(XP_UNIX) || defined(XP_OS2) || defined(XP_BEOS)
     // Some system calls require an EOF offset.
-    PRInt64 offset;
+    int64_t offset;
     rv = Tell(&offset);
     if (NS_FAILED(rv)) return rv;
 #endif
@@ -128,13 +133,13 @@ nsFileStreamBase::Close()
     if (mFD) {
         if (PR_Close(mFD) == PR_FAILURE)
             rv = NS_BASE_STREAM_OSERROR;
-        mFD = nsnull;
+        mFD = nullptr;
     }
     return rv;
 }
 
 nsresult
-nsFileStreamBase::Available(PRUint32* aResult)
+nsFileStreamBase::Available(uint64_t* aResult)
 {
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
@@ -145,18 +150,18 @@ nsFileStreamBase::Available(PRUint32* aResult)
 
     // PR_Available with files over 4GB returns an error, so we have to
     // use the 64-bit version of PR_Available.
-    PRInt64 avail = PR_Available64(mFD);
+    int64_t avail = PR_Available64(mFD);
     if (avail == -1) {
         return NS_ErrorAccordingToNSPR();
     }
 
     // If available is greater than 4GB, return 4GB
-    *aResult = avail > PR_UINT32_MAX ? PR_UINT32_MAX : (PRUint32)avail;
+    *aResult = (uint64_t)avail;
     return NS_OK;
 }
 
 nsresult
-nsFileStreamBase::Read(char* aBuf, PRUint32 aCount, PRUint32* aResult)
+nsFileStreamBase::Read(char* aBuf, uint32_t aCount, uint32_t* aResult)
 {
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
@@ -166,7 +171,7 @@ nsFileStreamBase::Read(char* aBuf, PRUint32 aCount, PRUint32* aResult)
         return NS_OK;
     }
 
-    PRInt32 bytesRead = PR_Read(mFD, aBuf, aCount);
+    int32_t bytesRead = PR_Read(mFD, aBuf, aCount);
     if (bytesRead == -1) {
         return NS_ErrorAccordingToNSPR();
     }
@@ -177,7 +182,7 @@ nsFileStreamBase::Read(char* aBuf, PRUint32 aCount, PRUint32* aResult)
 
 nsresult
 nsFileStreamBase::ReadSegments(nsWriteSegmentFun aWriter, void* aClosure,
-                               PRUint32 aCount, PRUint32* aResult)
+                               uint32_t aCount, uint32_t* aResult)
 {
     // ReadSegments is not implemented because it would be inefficient when
     // the writer does not consume all data.  If you want to call ReadSegments,
@@ -203,10 +208,10 @@ nsFileStreamBase::Flush(void)
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (mFD == nsnull)
+    if (mFD == nullptr)
         return NS_BASE_STREAM_CLOSED;
 
-    PRInt32 cnt = PR_Sync(mFD);
+    int32_t cnt = PR_Sync(mFD);
     if (cnt == -1) {
         return NS_ErrorAccordingToNSPR();
     }
@@ -214,15 +219,15 @@ nsFileStreamBase::Flush(void)
 }
 
 nsresult
-nsFileStreamBase::Write(const char *buf, PRUint32 count, PRUint32 *result)
+nsFileStreamBase::Write(const char *buf, uint32_t count, uint32_t *result)
 {
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (mFD == nsnull)
+    if (mFD == nullptr)
         return NS_BASE_STREAM_CLOSED;
 
-    PRInt32 cnt = PR_Write(mFD, buf, count);
+    int32_t cnt = PR_Write(mFD, buf, count);
     if (cnt == -1) {
         return NS_ErrorAccordingToNSPR();
     }
@@ -231,7 +236,7 @@ nsFileStreamBase::Write(const char *buf, PRUint32 count, PRUint32 *result)
 }
     
 nsresult
-nsFileStreamBase::WriteFrom(nsIInputStream *inStr, PRUint32 count, PRUint32 *_retval)
+nsFileStreamBase::WriteFrom(nsIInputStream *inStr, uint32_t count, uint32_t *_retval)
 {
     NS_NOTREACHED("WriteFrom (see source comment)");
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -241,7 +246,7 @@ nsFileStreamBase::WriteFrom(nsIInputStream *inStr, PRUint32 count, PRUint32 *_re
 }
 
 nsresult
-nsFileStreamBase::WriteSegments(nsReadSegmentFun reader, void * closure, PRUint32 count, PRUint32 *_retval)
+nsFileStreamBase::WriteSegments(nsReadSegmentFun reader, void * closure, uint32_t count, uint32_t *_retval)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
     // File streams intentionally do not support this method.
@@ -250,8 +255,8 @@ nsFileStreamBase::WriteSegments(nsReadSegmentFun reader, void * closure, PRUint3
 }
 
 nsresult
-nsFileStreamBase::MaybeOpen(nsIFile* aFile, PRInt32 aIoFlags,
-                            PRInt32 aPerm, bool aDeferred)
+nsFileStreamBase::MaybeOpen(nsIFile* aFile, int32_t aIoFlags,
+                            int32_t aPerm, bool aDeferred)
 {
     NS_ENSURE_STATE(aFile);
 
@@ -279,19 +284,23 @@ nsFileStreamBase::MaybeOpen(nsIFile* aFile, PRInt32 aIoFlags,
 void
 nsFileStreamBase::CleanUpOpen()
 {
-    mOpenParams.localFile = nsnull;
+    mOpenParams.localFile = nullptr;
     mDeferredOpen = false;
 }
 
 nsresult
 nsFileStreamBase::DoOpen()
 {
-    NS_PRECONDITION(mOpenParams.localFile, "Must have a file to open");
+    NS_ASSERTION(!mFD, "Already have a file descriptor!");
+    NS_ASSERTION(mOpenParams.localFile, "Must have a file to open");
 
     PRFileDesc* fd;
-    nsresult rv = mOpenParams.localFile->OpenNSPRFileDesc(mOpenParams.ioFlags, mOpenParams.perm, &fd);
+    nsresult rv = mOpenParams.localFile->OpenNSPRFileDesc(mOpenParams.ioFlags,
+                                                          mOpenParams.perm,
+                                                          &fd);
     CleanUpOpen();
-    if (NS_FAILED(rv)) return rv;
+    if (NS_FAILED(rv))
+        return rv;
     mFD = fd;
 
     return NS_OK;
@@ -320,16 +329,15 @@ NS_INTERFACE_MAP_BEGIN(nsFileInputStream)
     NS_INTERFACE_MAP_ENTRY(nsIInputStream)
     NS_INTERFACE_MAP_ENTRY(nsIFileInputStream)
     NS_INTERFACE_MAP_ENTRY(nsILineInputStream)
-    NS_INTERFACE_MAP_ENTRY(nsIIPCSerializable)
+    NS_INTERFACE_MAP_ENTRY(nsIIPCSerializableInputStream)
     NS_IMPL_QUERY_CLASSINFO(nsFileInputStream)
 NS_INTERFACE_MAP_END_INHERITING(nsFileStreamBase)
 
-NS_IMPL_CI_INTERFACE_GETTER5(nsFileInputStream,
+NS_IMPL_CI_INTERFACE_GETTER4(nsFileInputStream,
                              nsIInputStream,
                              nsIFileInputStream,
                              nsISeekableStream,
-                             nsILineInputStream,
-                             nsIIPCSerializable)
+                             nsILineInputStream)
 
 nsresult
 nsFileInputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
@@ -337,7 +345,7 @@ nsFileInputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
     NS_ENSURE_NO_AGGREGATION(aOuter);
 
     nsFileInputStream* stream = new nsFileInputStream();
-    if (stream == nsnull)
+    if (stream == nullptr)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(stream);
     nsresult rv = stream->QueryInterface(aIID, aResult);
@@ -346,7 +354,7 @@ nsFileInputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
 }
 
 nsresult
-nsFileInputStream::Open(nsIFile* aFile, PRInt32 aIOFlags, PRInt32 aPerm)
+nsFileInputStream::Open(nsIFile* aFile, int32_t aIOFlags, int32_t aPerm)
 {   
     nsresult rv = NS_OK;
 
@@ -383,8 +391,8 @@ nsFileInputStream::Open(nsIFile* aFile, PRInt32 aIOFlags, PRInt32 aPerm)
 }
 
 NS_IMETHODIMP
-nsFileInputStream::Init(nsIFile* aFile, PRInt32 aIOFlags, PRInt32 aPerm,
-                        PRInt32 aBehaviorFlags)
+nsFileInputStream::Init(nsIFile* aFile, int32_t aIOFlags, int32_t aPerm,
+                        int32_t aBehaviorFlags)
 {
     NS_ENSURE_TRUE(!mFD, NS_ERROR_ALREADY_INITIALIZED);
     NS_ENSURE_TRUE(!mDeferredOpen, NS_ERROR_ALREADY_INITIALIZED);
@@ -410,14 +418,14 @@ nsFileInputStream::Close()
         NS_ASSERTION(NS_SUCCEEDED(rv), "failed to delete file");
         // If we don't need to save the file for reopening, free it up
         if (!(mBehaviorFlags & REOPEN_ON_REWIND)) {
-          mFile = nsnull;
+          mFile = nullptr;
         }
     }
     return rv;
 }
 
 NS_IMETHODIMP
-nsFileInputStream::Read(char* aBuf, PRUint32 aCount, PRUint32* _retval)
+nsFileInputStream::Read(char* aBuf, uint32_t aCount, uint32_t* _retval)
 {
     nsresult rv = nsFileStreamBase::Read(aBuf, aCount, _retval);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -444,7 +452,7 @@ nsFileInputStream::ReadLine(nsACString& aLine, bool* aResult)
 }
 
 NS_IMETHODIMP
-nsFileInputStream::Seek(PRInt32 aWhence, PRInt64 aOffset)
+nsFileInputStream::Seek(int32_t aWhence, int64_t aOffset)
 {
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
@@ -464,56 +472,99 @@ nsFileInputStream::Seek(PRInt32 aWhence, PRInt64 aOffset)
     return nsFileStreamBase::Seek(aWhence, aOffset);
 }
 
-bool
-nsFileInputStream::Read(const IPC::Message *aMsg, void **aIter)
+void
+nsFileInputStream::Serialize(InputStreamParams& aParams)
 {
-    using IPC::ReadParam;
+    FileInputStreamParams params;
 
-    nsCString path;
-    bool followLinks;
-    PRInt32 flags;
-    if (!ReadParam(aMsg, aIter, &path) ||
-        !ReadParam(aMsg, aIter, &followLinks) ||
-        !ReadParam(aMsg, aIter, &flags))
-        return false;
+    if (mFD) {
+        FileHandleType fd = FileHandleType(PR_FileDesc2NativeHandle(mFD));
+        NS_ASSERTION(fd, "This should never be null!");
 
-    nsCOMPtr<nsIFile> file;
-    nsresult rv = NS_NewNativeLocalFile(path, followLinks, getter_AddRefs(file));
-    if (NS_FAILED(rv))
-        return false;
+        params.file() = FileDescriptor(fd);
+        NS_ASSERTION(params.file().IsValid(),
+                     "Sending an invalid file descriptor!");
+    } else {
+        NS_WARNING("This file has not been opened (or could not be opened). "
+                   "Sending an invalid file descriptor to the other process!");
+    }
 
-    // IO flags = -1 means readonly, and
-    // permissions are unimportant since we're reading
-    rv = Init(file, -1, -1, flags);
-    if (NS_FAILED(rv))
-        return false;
+    int32_t behaviorFlags = mBehaviorFlags;
 
-    return true;
+    // The other process shouldn't close when it reads the end because it will
+    // not be able to reopen the file later.
+    behaviorFlags &= ~nsIFileInputStream::CLOSE_ON_EOF;
+
+    // The other process will not be able to reopen the file so transferring
+    // this flag is meaningless.
+    behaviorFlags &= ~nsIFileInputStream::REOPEN_ON_REWIND;
+
+    // The other process is going to have an open file descriptor automatically
+    // so transferring this flag is meaningless.
+    behaviorFlags &= ~nsIFileInputStream::DEFER_OPEN;
+
+    params.behaviorFlags() = behaviorFlags;
+    params.ioFlags() = mIOFlags;
+
+    aParams = params;
 }
 
-void
-nsFileInputStream::Write(IPC::Message *aMsg)
+bool
+nsFileInputStream::Deserialize(const InputStreamParams& aParams)
 {
-    using IPC::WriteParam;
+    NS_ASSERTION(!mFD, "Already have a file descriptor?!");
+    NS_ASSERTION(!mDeferredOpen, "Deferring open?!");
+    NS_ASSERTION(!mFile, "Should never have a file here!");
+    NS_ASSERTION(!mPerm, "This should always be 0!");
 
-    nsCString path;
-    mFile->GetNativePath(path);
-    WriteParam(aMsg, path);
-    bool followLinks;
-    mFile->GetFollowLinks(&followLinks);
-    WriteParam(aMsg, followLinks);
-    WriteParam(aMsg, mBehaviorFlags);
+    if (aParams.type() != InputStreamParams::TFileInputStreamParams) {
+        NS_WARNING("Received unknown parameters from the other process!");
+        return false;
+    }
+
+    const FileInputStreamParams& params = aParams.get_FileInputStreamParams();
+
+    const FileDescriptor& fd = params.file();
+    NS_WARN_IF_FALSE(fd.IsValid(), "Received an invalid file descriptor!");
+
+    if (fd.IsValid()) {
+        PRFileDesc* fileDesc = PR_ImportFile(PROsfd(fd.PlatformHandle()));
+        if (!fileDesc) {
+            NS_WARNING("Failed to import file handle!");
+            return false;
+        }
+        mFD = fileDesc;
+    }
+
+    mBehaviorFlags = params.behaviorFlags();
+    mIOFlags = params.ioFlags();
+
+    return true;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // nsPartialFileInputStream
 
+NS_IMPL_ADDREF_INHERITED(nsPartialFileInputStream, nsFileStreamBase)
+NS_IMPL_RELEASE_INHERITED(nsPartialFileInputStream, nsFileStreamBase)
+
+NS_IMPL_CLASSINFO(nsPartialFileInputStream, NULL, nsIClassInfo::THREADSAFE,
+                  NS_PARTIALLOCALFILEINPUTSTREAM_CID)
+
 // Don't forward to nsFileInputStream as we don't want to QI to
 // nsIFileInputStream
-NS_IMPL_ISUPPORTS_INHERITED3(nsPartialFileInputStream,
-                             nsFileStreamBase,
+NS_INTERFACE_MAP_BEGIN(nsPartialFileInputStream)
+    NS_INTERFACE_MAP_ENTRY(nsIInputStream)
+    NS_INTERFACE_MAP_ENTRY(nsIPartialFileInputStream)
+    NS_INTERFACE_MAP_ENTRY(nsILineInputStream)
+    NS_INTERFACE_MAP_ENTRY(nsIIPCSerializableInputStream)
+    NS_IMPL_QUERY_CLASSINFO(nsPartialFileInputStream)
+NS_INTERFACE_MAP_END_INHERITING(nsFileStreamBase)
+
+NS_IMPL_CI_INTERFACE_GETTER4(nsPartialFileInputStream,
                              nsIInputStream,
                              nsIPartialFileInputStream,
+                             nsISeekableStream,
                              nsILineInputStream)
 
 nsresult
@@ -531,9 +582,9 @@ nsPartialFileInputStream::Create(nsISupports *aOuter, REFNSIID aIID,
 }
 
 NS_IMETHODIMP
-nsPartialFileInputStream::Init(nsIFile* aFile, PRUint64 aStart,
-                               PRUint64 aLength, PRInt32 aIOFlags,
-                               PRInt32 aPerm, PRInt32 aBehaviorFlags)
+nsPartialFileInputStream::Init(nsIFile* aFile, uint64_t aStart,
+                               uint64_t aLength, int32_t aIOFlags,
+                               int32_t aPerm, int32_t aBehaviorFlags)
 {
     mStart = aStart;
     mLength = aLength;
@@ -547,9 +598,9 @@ nsPartialFileInputStream::Init(nsIFile* aFile, PRUint64 aStart,
 }
 
 NS_IMETHODIMP
-nsPartialFileInputStream::Tell(PRInt64 *aResult)
+nsPartialFileInputStream::Tell(int64_t *aResult)
 {
-    PRInt64 tell;
+    int64_t tell;
     nsresult rv = nsFileInputStream::Tell(&tell);
     if (NS_SUCCEEDED(rv)) {
         *aResult = tell - mStart;
@@ -558,9 +609,9 @@ nsPartialFileInputStream::Tell(PRInt64 *aResult)
 }
 
 NS_IMETHODIMP
-nsPartialFileInputStream::Available(PRUint32* aResult)
+nsPartialFileInputStream::Available(uint64_t* aResult)
 {
-    PRUint32 available;
+    uint64_t available;
     nsresult rv = nsFileInputStream::Available(&available);
     if (NS_SUCCEEDED(rv)) {
         *aResult = TruncateSize(available);
@@ -569,9 +620,9 @@ nsPartialFileInputStream::Available(PRUint32* aResult)
 }
 
 NS_IMETHODIMP
-nsPartialFileInputStream::Read(char* aBuf, PRUint32 aCount, PRUint32* aResult)
+nsPartialFileInputStream::Read(char* aBuf, uint32_t aCount, uint32_t* aResult)
 {
-    PRUint32 readsize = TruncateSize(aCount);
+    uint32_t readsize = (uint32_t) TruncateSize(aCount);
     if (readsize == 0 && mBehaviorFlags & CLOSE_ON_EOF) {
         Close();
         *aResult = 0;
@@ -586,9 +637,9 @@ nsPartialFileInputStream::Read(char* aBuf, PRUint32 aCount, PRUint32* aResult)
 }
 
 NS_IMETHODIMP
-nsPartialFileInputStream::Seek(PRInt32 aWhence, PRInt64 aOffset)
+nsPartialFileInputStream::Seek(int32_t aWhence, int64_t aOffset)
 {
-    PRInt64 offset;
+    int64_t offset;
     switch (aWhence) {
         case NS_SEEK_SET:
             offset = mStart + aOffset;
@@ -603,7 +654,7 @@ nsPartialFileInputStream::Seek(PRInt32 aWhence, PRInt64 aOffset)
             return NS_ERROR_ILLEGAL_VALUE;
     }
 
-    if (offset < (PRInt64)mStart || offset > (PRInt64)(mStart + mLength)) {
+    if (offset < (int64_t)mStart || offset > (int64_t)(mStart + mLength)) {
         return NS_ERROR_INVALID_ARG;
     }
 
@@ -614,6 +665,64 @@ nsPartialFileInputStream::Seek(PRInt32 aWhence, PRInt64 aOffset)
     return rv;
 }
 
+void
+nsPartialFileInputStream::Serialize(InputStreamParams& aParams)
+{
+    // Serialize the base class first.
+    InputStreamParams fileParams;
+    nsFileInputStream::Serialize(fileParams);
+
+    if (fileParams.type() != InputStreamParams::TFileInputStreamParams) {
+        NS_ERROR("Base class serialize failed!");
+        return;
+    }
+
+    PartialFileInputStreamParams params;
+
+    params.fileStreamParams() = fileParams.get_FileInputStreamParams();
+    params.begin() = mStart;
+    params.length() = mLength;
+
+    aParams = params;
+}
+
+bool
+nsPartialFileInputStream::Deserialize(const InputStreamParams& aParams)
+{
+    NS_ASSERTION(!mFD, "Already have a file descriptor?!");
+    NS_ASSERTION(!mStart, "Already have a start?!");
+    NS_ASSERTION(!mLength, "Already have a length?!");
+    NS_ASSERTION(!mPosition, "Already have a position?!");
+
+    if (aParams.type() != InputStreamParams::TPartialFileInputStreamParams) {
+        NS_ERROR("Received unknown parameters from the other process!");
+        return false;
+    }
+
+    const PartialFileInputStreamParams& params =
+        aParams.get_PartialFileInputStreamParams();
+
+    // Deserialize the base class first.
+    InputStreamParams fileParams(params.fileStreamParams());
+    if (!nsFileInputStream::Deserialize(fileParams)) {
+        NS_ERROR("Base class deserialize failed!");
+        return false;
+    }
+
+    NS_ASSERTION(mFD, "Must have a file descriptor now!");
+
+    mStart = params.begin();
+    mLength = params.length();
+    mPosition = 0;
+
+    if (!mStart) {
+      return true;
+    }
+
+    // XXX This is so broken. Main thread IO alert.
+    return NS_SUCCEEDED(nsFileInputStream::Seek(NS_SEEK_SET, mStart));
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // nsFileOutputStream
 
@@ -628,7 +737,7 @@ nsFileOutputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
     NS_ENSURE_NO_AGGREGATION(aOuter);
 
     nsFileOutputStream* stream = new nsFileOutputStream();
-    if (stream == nsnull)
+    if (stream == nullptr)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(stream);
     nsresult rv = stream->QueryInterface(aIID, aResult);
@@ -637,10 +746,10 @@ nsFileOutputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
 }
 
 NS_IMETHODIMP
-nsFileOutputStream::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm,
-                         PRInt32 behaviorFlags)
+nsFileOutputStream::Init(nsIFile* file, int32_t ioFlags, int32_t perm,
+                         int32_t behaviorFlags)
 {
-    NS_ENSURE_TRUE(mFD == nsnull, NS_ERROR_ALREADY_INITIALIZED);
+    NS_ENSURE_TRUE(mFD == nullptr, NS_ERROR_ALREADY_INITIALIZED);
     NS_ENSURE_TRUE(!mDeferredOpen, NS_ERROR_ALREADY_INITIALIZED);
 
     mBehaviorFlags = behaviorFlags;
@@ -664,8 +773,8 @@ NS_IMPL_ISUPPORTS_INHERITED3(nsSafeFileOutputStream,
                              nsIFileOutputStream)
 
 NS_IMETHODIMP
-nsSafeFileOutputStream::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm,
-                             PRInt32 behaviorFlags)
+nsSafeFileOutputStream::Init(nsIFile* file, int32_t ioFlags, int32_t perm,
+                             int32_t behaviorFlags)
 {
     return nsFileOutputStream::Init(file, ioFlags, perm, behaviorFlags);
 }
@@ -698,7 +807,7 @@ nsSafeFileOutputStream::DoOpen()
     }
 
     if (NS_SUCCEEDED(rv) && mTargetFileExists) {
-        PRUint32 origPerm;
+        uint32_t origPerm;
         if (NS_FAILED(file->GetPermissions(&origPerm))) {
             NS_ERROR("Can't get permissions of target file");
             origPerm = mOpenParams.perm;
@@ -727,7 +836,7 @@ nsSafeFileOutputStream::Close()
     // so clean up by removing the temp file.
     if (mTempFile) {
         mTempFile->Remove(false);
-        mTempFile = nsnull;
+        mTempFile = nullptr;
     }
 
     return rv;
@@ -764,7 +873,7 @@ nsSafeFileOutputStream::Finish()
             rv = mTargetFile->GetLeafName(targetFilename);
             if (NS_SUCCEEDED(rv)) {
                 // This will replace target.
-                rv = mTempFile->MoveTo(nsnull, targetFilename);
+                rv = mTempFile->MoveTo(nullptr, targetFilename);
                 if (NS_FAILED(rv))
                     mTempFile->Remove(false);
             }
@@ -777,12 +886,12 @@ nsSafeFileOutputStream::Finish()
         if (NS_FAILED(mWriteResult))
             rv = mWriteResult;
     }
-    mTempFile = nsnull;
+    mTempFile = nullptr;
     return rv;
 }
 
 NS_IMETHODIMP
-nsSafeFileOutputStream::Write(const char *buf, PRUint32 count, PRUint32 *result)
+nsSafeFileOutputStream::Write(const char *buf, uint32_t count, uint32_t *result)
 {
     nsresult rv = nsFileOutputStream::Write(buf, count, result);
     if (NS_SUCCEEDED(mWriteResult)) {
@@ -808,10 +917,10 @@ NS_IMPL_ISUPPORTS_INHERITED4(nsFileStream,
                              nsIFileMetadata)
  
 NS_IMETHODIMP
-nsFileStream::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm,
-                   PRInt32 behaviorFlags)
+nsFileStream::Init(nsIFile* file, int32_t ioFlags, int32_t perm,
+                   int32_t behaviorFlags)
 {
-    NS_ENSURE_TRUE(mFD == nsnull, NS_ERROR_ALREADY_INITIALIZED);
+    NS_ENSURE_TRUE(mFD == nullptr, NS_ERROR_ALREADY_INITIALIZED);
     NS_ENSURE_TRUE(!mDeferredOpen, NS_ERROR_ALREADY_INITIALIZED);
 
     mBehaviorFlags = behaviorFlags;
@@ -826,7 +935,7 @@ nsFileStream::Init(nsIFile* file, PRInt32 ioFlags, PRInt32 perm,
 }
 
 NS_IMETHODIMP
-nsFileStream::GetSize(PRInt64* _retval)
+nsFileStream::GetSize(int64_t* _retval)
 {
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
@@ -840,13 +949,13 @@ nsFileStream::GetSize(PRInt64* _retval)
         return NS_BASE_STREAM_OSERROR;
     }
 
-    *_retval = PRInt64(info.size);
+    *_retval = int64_t(info.size);
 
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileStream::GetLastModified(PRInt64* _retval)
+nsFileStream::GetLastModified(int64_t* _retval)
 {
     nsresult rv = DoPendingOpen();
     NS_ENSURE_SUCCESS(rv, rv);
@@ -860,15 +969,15 @@ nsFileStream::GetLastModified(PRInt64* _retval)
         return NS_BASE_STREAM_OSERROR;
     }
 
-    PRInt64 modTime = PRInt64(info.modifyTime);
+    int64_t modTime = int64_t(info.modifyTime);
     if (modTime == 0) {
         *_retval = 0;
     }
     else {
-        *_retval = modTime / PRInt64(PR_USEC_PER_MSEC);
+        *_retval = modTime / int64_t(PR_USEC_PER_MSEC);
     }
 
     return NS_OK;
 }
 
-////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
\ No newline at end of file
