@@ -30,6 +30,9 @@
 #include "nsDirectoryServiceUtils.h"
 #include "nsDirectoryServiceDefs.h"
 
+// JS
+#include "jsdbgapi.h"
+
 // we eventually want to make this runtime switchable
 #if defined(MOZ_PROFILING) && (defined(XP_UNIX) && !defined(XP_MACOSX))
  #ifndef ANDROID
@@ -129,6 +132,11 @@ public:
     , mTagName(aTagName)
   { }
 
+  ProfileEntry(char aTagName, int aTagLine)
+    : mTagLine(aTagLine)
+    , mTagName(aTagName)
+  { }
+
   friend std::ostream& operator<<(std::ostream& stream, const ProfileEntry& entry);
 
 private:
@@ -140,6 +148,7 @@ private:
     double mTagFloat;
     Address mTagAddress;
     uintptr_t mTagOffset;
+    int mTagLine;
   };
   char mTagName;
 };
@@ -331,6 +340,13 @@ public:
                 b.DefineProperty(frame, "location", tagBuff);
               } else {
                 b.DefineProperty(frame, "location", tagStringData);
+                readAheadPos = (readPos + incBy) % mEntrySize;
+                if (readAheadPos != mLastFlushPos &&
+                    mEntries[readAheadPos].mTagName == 'n') {
+                  b.DefineProperty(frame, "line",
+                                   mEntries[readAheadPos].mTagLine);
+                  incBy++;
+                }
               }
               b.ArrayPush(frames, frame);
             }
@@ -454,6 +470,10 @@ public:
     t->SetPaused(true);
 
     // Get file path
+#ifdef ANDROID
+    nsCString tmpPath;
+    tmpPath.AppendPrintf("/sdcard/profile_%i_%i.txt", XRE_GetProcessType(), getpid());
+#else
     nsCOMPtr<nsIFile> tmpFile;
     nsCAutoString tmpPath;
     if (NS_FAILED(NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(tmpFile)))) {
@@ -469,6 +489,7 @@ public:
     rv = tmpFile->GetNativePath(tmpPath);
     if (NS_FAILED(rv))
       return rv;
+#endif
 
     // Create a JSContext to run a JSObjectBuilder :(
     // Based on XPCShellEnvironment
@@ -504,14 +525,10 @@ public:
       // being thread safe. Bug 750989.
       t->SetPaused(true);
       if (stream.is_open()) {
-        JSAutoEnterCompartment autoComp;
-        if (autoComp.enter(cx, obj)) {
-          JSObject* profileObj = mozilla_sampler_get_profile_data(cx);
-          jsval val = OBJECT_TO_JSVAL(profileObj);
-          JS_Stringify(cx, &val, nsnull, JSVAL_NULL, WriteCallback, &stream);
-        } else {
-          LOG("Failed to enter compartment");
-        }
+        JSAutoCompartment autoComp(cx, obj);
+        JSObject* profileObj = mozilla_sampler_get_profile_data(cx);
+        jsval val = OBJECT_TO_JSVAL(profileObj);
+        JS_Stringify(cx, &val, nullptr, JSVAL_NULL, WriteCallback, &stream);
         stream.close();
         LOGF("Saved to %s", tmpPath.get());
       } else {
@@ -599,7 +616,6 @@ JSObject* TableTicker::ToJSObject(JSContext *aCx)
   JSObject *profile = b.CreateObject();
 
   // Put shared library info
-  JSObject *libs = GetMetaJSObject(b);
   b.DefineProperty(profile, "libs", GetSharedLibraryInfoString().c_str());
 
   // Put meta data
@@ -620,12 +636,15 @@ JSObject* TableTicker::ToJSObject(JSContext *aCx)
 }
 
 static
-void addProfileEntry(ProfileStack *aStack, ThreadProfile &aProfile, int i)
+void addProfileEntry(volatile StackEntry &entry, ThreadProfile &aProfile,
+                     ProfileStack *stack, void *lastpc)
 {
+  int lineno = -1;
+
   // First entry has tagName 's' (start)
   // Check for magic pointer bit 1 to indicate copy
-  const char* sampleLabel = aStack->mStack[i].mLabel;
-  if (aStack->mStack[i].isCopyLabel()) {
+  const char* sampleLabel = entry.label();
+  if (entry.isCopyLabel()) {
     // Store the string using 1 or more 'd' (dynamic) tags
     // that will happen to the preceding tag
 
@@ -642,8 +661,30 @@ void addProfileEntry(ProfileStack *aStack, ThreadProfile &aProfile, int i)
       // Cast to *((void**) to pass the text data to a void*
       aProfile.addTag(ProfileEntry('d', *((void**)(&text[0]))));
     }
+    if (entry.js()) {
+      if (!entry.pc()) {
+        // The JIT only allows the top-most entry to have a NULL pc
+        MOZ_ASSERT(&entry == &stack->mStack[stack->stackSize() - 1]);
+        // If stack-walking was disabled, then that's just unfortunate
+        if (lastpc) {
+          jsbytecode *jspc = js::ProfilingGetPC(stack->mRuntime, entry.script(),
+                                                lastpc);
+          if (jspc) {
+            lineno = JS_PCToLineNumber(NULL, entry.script(), jspc);
+          }
+        }
+      } else {
+        lineno = JS_PCToLineNumber(NULL, entry.script(), entry.pc());
+      }
+    } else {
+      lineno = entry.line();
+    }
   } else {
     aProfile.addTag(ProfileEntry('c', sampleLabel));
+    lineno = entry.line();
+  }
+  if (lineno != -1) {
+    aProfile.addTag(ProfileEntry('n', lineno));
   }
 }
 
@@ -736,13 +777,13 @@ void TableTicker::doBacktrace(ThreadProfile &aProfile, TickSample* aSample)
     // pseudoStackPos is the position in the Pseudo stack starting
     // at the first frame (run_js in the example) and increasing.
     for (size_t i = array.count; i > 0; --i) {
-      while (pseudoStackPos < stack->mStackPointer) {
+      while (pseudoStackPos < stack->stackSize()) {
         volatile StackEntry& entry = stack->mStack[pseudoStackPos];
 
-        if (entry.mStackAddress < array.sp_array[i-1] && entry.mStackAddress)
+        if (entry.stackAddress() < array.sp_array[i-1] && entry.stackAddress())
           break;
 
-        addProfileEntry(stack, aProfile, pseudoStackPos);
+        addProfileEntry(entry, aProfile, stack, array.array[0]);
         pseudoStackPos++;
       }
 
@@ -807,10 +848,8 @@ void doSampleStackTrace(ProfileStack *aStack, ThreadProfile &aProfile, TickSampl
   // 's' tag denotes the start of a sample block
   // followed by 0 or more 'c' tags.
   aProfile.addTag(ProfileEntry('s', "(root)"));
-  for (mozilla::sig_safe_t i = 0;
-       i < aStack->mStackPointer && i < mozilla::ArrayLength(aStack->mStack);
-       i++) {
-    addProfileEntry(aStack, aProfile, i);
+  for (uint32_t i = 0; i < aStack->stackSize(); i++) {
+    addProfileEntry(aStack->mStack[i], aProfile, aStack, nullptr);
   }
 #ifdef ENABLE_SPS_LEAF_DATA
   if (sample) {
